%if 0%{?rhel} < 7
%{!?__global_ldflags: %global __global_ldflags -Wl,-z,relro}
%endif

%bcond_without doc

Name: qr_mumps
Version: 1.2
Release: 3%{?dist}
Summary: A multithreaded multifrontal QR solver
License: LGPLv3
Group: Development/Libraries
URL: http://buttari.perso.enseeiht.fr/qr_mumps/
Source0: http://buttari.perso.enseeiht.fr/qr_mumps/releases/%{version}/qr_mumps-%{version}.tgz

# Custom Makefile changed for Fedora and built from Make.inc/Makefile.gfortran.PAR in the source.
Source1: %{name}-Makefile.inc

BuildRequires: gcc-gfortran
BuildRequires: blas-devel
BuildRequires: lapack-devel
BuildRequires: metis-devel >= 5.1.0-12
BuildRequires: scotch-devel
BuildRequires: perl
%if %{with doc}
BuildRequires: doxygen
%endif

Requires: gcc-gfortran%{?_isa}

%description
qr_mumps is a software package for the solution of sparse,
linear systems on multicore computers.
It implements a direct solution method based on the QR
factorization of the input matrix. Therefore, it is suited
to solving sparse least-squares problems and to computing
the minimum-norm solution of sparse, underdetermined problems.
It can obviously be used for solving square problems in which
case the stability provided by the use of orthogonal transformations
comes at the cost of a higher operation count with respect to solvers
based on, e.g., the LU factorization.
qr_mumps supports real and complex, single or double precision arithmetic.

%package devel
Summary: Development files for %{name}
Group: Development/Libraries
Requires: %{name}%{?_isa} = %{version}-%{release}
%description devel
Shared links and header files of %{name}.

%package openmp
Summary: %{name} with OpenMP support
Group: Development/Libraries
Requires: gcc-gfortran%{?_isa}
%description openmp
%{name} with OpenMP support.

%package openmp-devel
Summary: Development files for %{name}
Group: Development/Libraries
Requires: %{name}-openmp%{?_isa} = %{version}-%{release}
%description openmp-devel
Shared links and header files of %{name}.

%if %{with doc}
%package doc
Summary: HTML/PDF documentation files of %{name}
Group: Development/Libraries
BuildArch: noarch
%description doc
HTML/PDF documentation files of %{name}.
%endif

########################################################

%prep
%setup -q -n %{name}-%{version}

cp -p %{SOURCE1} Make.inc

%build

## Build serial version
make \
 TOPDIR=./ \
 CC=gcc \
 FC=gfortran \
 FCFLAGS="%{optflags} -fPIC -Wl,-z,now" \
 CFLAGS="%{optflags} -fPIC -Wl,-z,now" \
 CDEFS=" -Dhave_metis -Dhave_scotch" \
 FDEFS=" -Dhave_metis -Dhave_scotch" \
 LCOLAMD="" \
 ICOLAMD="" \
 LMETIS=" -lmetis" \
 IMETIS=" -I%{_includedir}" \
 LSCOTCH=" -lscotch -lscotcherr" \
 ISCOTCH=" -I%{_includedir}"

# Make shared libraries
pushd lib
gfortran -shared %{__global_ldflags} -fPIC -Wl,-z,now -Wl,--whole-archive libqrm_common.a -Wl,-no-whole-archive -Wl,--as-needed -L%{_libdir} -lmetis -lgfortran -lm -lrt -Wl,-soname,libqrm_common.so.%{version} -o libqrm_common.so.%{version}

gfortran -shared %{__global_ldflags} -fPIC -Wl,-z,now -Wl,--whole-archive libdqrm.a libqrm_common.a -Wl,-no-whole-archive -Wl,--as-needed -L./ -lqrm_common -L%{_libdir} -lmetis -lgfortran -lm -lrt -Wl,-soname,libdqrm.so.%{version} -o libdqrm.so.%{version}
popd

cp -a lib lib-serial
cp -a include include-serial
rm -f lib/*

################# Cleaning ##########################
make clean
#####################################################

## Build fopenmp version
make \
 TOPDIR=./ \
 CC=gcc \
 FC=gfortran \
 FCFLAGS="%{optflags} -fopenmp -fPIC -Wl,-z,now" \
 CFLAGS="%{optflags} -fopenmp -fPIC -Wl,-z,now" \
 CDEFS=" -Dhave_metis -Dhave_scotch" \
 FDEFS=" -Dhave_metis -Dhave_scotch" \
 LCOLAMD="" \
 ICOLAMD="" \
 LMETIS=" -lmetis" \
 IMETIS=" -I%{_includedir}" \
 LSCOTCH=" -lscotch -lscotcherr" \
 ISCOTCH=" -I%{_includedir}"

# Make shared libraries
pushd lib
gfortran -shared %{__global_ldflags} -fPIC -fopenmp -Wl,-z,now -Wl,--whole-archive libqrm_common.a -Wl,-no-whole-archive -Wl,--as-needed -L%{_libdir} -lmetis -lgfortran -lm -lrt -Wl,-soname,libqrmMP_common.so.%{version} -o libqrmMP_common.so.%{version}

gfortran -shared %{__global_ldflags} -fPIC -fopenmp -Wl,-z,now -Wl,--whole-archive libdqrm.a libqrm_common.a -Wl,-no-whole-archive -Wl,--as-needed -L./ -lqrm_common -L%{_libdir} -lmetis -lgfortran -lm -lrt -Wl,-soname,libdqrmMP.so.%{version} -o libdqrmMP.so.%{version}
popd

cp -a lib lib-openmp
cp -a include include-openmp

################# Cleaning ##########################
make clean
#####################################################

%if %{with doc}
pushd doc/Doxygen
doxygen qrm_doxyconf
%endif
#######################################################

%post -p /sbin/ldconfig
%postun -p /sbin/ldconfig

%post -n qr_mumps-openmp -p /sbin/ldconfig
%postun -n qr_mumps-openmp -p /sbin/ldconfig

%check
pushd test
make \
 FCFLAGS="%{optflags} -fopenmp -fPIC -Wl,-z,now" \
 CFLAGS="%{optflags} -fopenmp -fPIC -Wl,-z,now" \
 CDEFS=" -Dhave_metis -Dhave_scotch" \
 FDEFS=" -Dhave_metis -Dhave_scotch" \
 LCOLAMD="" \
 ICOLAMD="" \
 LMETIS=" -lmetis" \
 IMETIS=" -I%{_includedir}" \
 LSCOTCH=" -lscotch -lscotcherr" \
 ISCOTCH=" -I%{_includedir}" \
 QRM_LIB=$RPM_BUILD_ROOT%{_libdir}/libdqrmMP.so.%{version} QRM_COMMONLIB=$RPM_BUILD_ROOT%{_libdir}/libqrmMP_common.so.%{version}
export LD_LIBRARY_PATH=$RPM_BUILD_ROOT%{_libdir}:%{_libdir}
%ifarch ppc ppc64 ppc64le
./dqrm_coverage 2>&1
%else
./dqrm_coverage
%endif

################# Cleaning ##########################
make clean
#####################################################

make \
 FCFLAGS="%{optflags} -fPIC -Wl,-z,now" \
 CFLAGS="%{optflags} -fPIC -Wl,-z,now" \
 CDEFS=" -Dhave_metis -Dhave_scotch" \
 FDEFS=" -Dhave_metis -Dhave_scotch" \
 LCOLAMD="" \
 ICOLAMD="" \
 LMETIS=" -lmetis" \
 IMETIS=" -I%{_includedir}" \
 LSCOTCH=" -lscotch -lscotcherr" \
 ISCOTCH=" -I%{_includedir}" \
 QRM_LIB=$RPM_BUILD_ROOT%{_libdir}/libdqrm.so.%{version} QRM_COMMONLIB=$RPM_BUILD_ROOT%{_libdir}/libqrm_common.so.%{version}
export LD_LIBRARY_PATH=$RPM_BUILD_ROOT%{_libdir}:%{_libdir}
%ifarch ppc ppc64 ppc64le
./dqrm_coverage 2>&1
%else
./dqrm_coverage
%endif
popd

%install
rm -rf $RPM_BUILD_ROOT
# Install libraries
mkdir -p $RPM_BUILD_ROOT%{_includedir}/%{name}
mkdir -p $RPM_BUILD_ROOT%{_includedir}/%{name}-openmp
mkdir -p $RPM_BUILD_ROOT%{_fmoddir}/%{name}
mkdir -p $RPM_BUILD_ROOT%{_fmoddir}/%{name}-openmp
pushd lib-serial
install -pm 755 *.so.%{version} $RPM_BUILD_ROOT%{_libdir}

ln -sf %{_libdir}/libqrm_common.so.%{version} $RPM_BUILD_ROOT%{_libdir}/libqrm_common.so.1
ln -sf %{_libdir}/libqrm_common.so.%{version} $RPM_BUILD_ROOT%{_libdir}/libqrm_common.so
ln -sf %{_libdir}/libdqrm.so.%{version} $RPM_BUILD_ROOT%{_libdir}/libdqrm.so.1
ln -sf %{_libdir}/libdqrm.so.%{version} $RPM_BUILD_ROOT%{_libdir}/libdqrm.so
popd

pushd lib-openmp
install -pm 755 *.so.%{version} $RPM_BUILD_ROOT%{_libdir}

ln -sf %{_libdir}/libqrmMP_common.so.%{version} $RPM_BUILD_ROOT%{_libdir}/libqrmMP_common.so.1
ln -sf %{_libdir}/libqrmMP_common.so.%{version} $RPM_BUILD_ROOT%{_libdir}/libqrmMP_common.so
ln -sf %{_libdir}/libdqrmMP.so.%{version} $RPM_BUILD_ROOT%{_libdir}/libdqrmMP.so.1
ln -sf %{_libdir}/libdqrmMP.so.%{version} $RPM_BUILD_ROOT%{_libdir}/libdqrmMP.so
popd

install -pm 644 include-serial/*.h $RPM_BUILD_ROOT%{_includedir}/%{name}
# Install Fortran modules
install -pm 644 include-serial/*.mod $RPM_BUILD_ROOT%{_fmoddir}/%{name}/

install -pm 644 include-openmp/*.h $RPM_BUILD_ROOT%{_includedir}/%{name}-openmp
# Install Fortran modules
install -pm 644 include-openmp/*.mod $RPM_BUILD_ROOT%{_fmoddir}/%{name}-openmp

%files
%license doc/COPYING.LESSER
%doc Changelog.org
%{_libdir}/libdqrm.so.*
%{_libdir}/libqrm_common.so.*

%files devel
%{_includedir}/%{name}/
%{_fmoddir}/%{name}/
%{_libdir}/libdqrm.so
%{_libdir}/libqrm_common.so

%files openmp
%license doc/COPYING.LESSER
%doc Changelog.org
%{_libdir}/libdqrmMP.so.*
%{_libdir}/libqrmMP_common.so.*

%files openmp-devel
%{_includedir}/%{name}-openmp/
%{_fmoddir}/%{name}-openmp/
%{_libdir}/libdqrmMP.so
%{_libdir}/libqrmMP_common.so

%if %{with doc}
%files doc
%license doc/COPYING.LESSER
%doc doc/Doxygen/html doc/pdf/*.pdf
%endif

%changelog
* Mon Mar 07 2016 Antonio Trande <sagitterATfedoraproject.org> - 1.2-3
- Fixed ln commands

* Fri Feb 19 2016 Antonio Trande <sagitterATfedoraproject.org> - 1.2-2
- Use Conditional Builds macro
- Remove pkgconfig BR

* Sat Feb 13 2016 Antonio Trande <sagitterATfedoraproject.org> - 1.2-1
- Initial package
